% This module performs the iteration step for an automatic theorem prover for
% a sequent system for first-order classical logic.

module lc_iter.

import  lc_prove.

type    add_copies      int -> form -> (list form) -> (list form) -> o.
type    amplify_forall  int -> (list form) -> (list form) -> o.
type    amplify_exists  int -> (list form) -> (list form) -> o.
type    amplify         int -> seq -> seq -> o.
type    nprove          int -> lprf -> seq -> o.

add_copies 1 A Lis (A::Lis).
add_copies N A Lis (A::Lis1) :-
  (N > 1), M is (N - 1),
  add_copies M A Lis Lis1.

amplify_forall N nil nil.
amplify_forall N ((forall A)::Gamma) Gamma2 :-
  amplify_forall N Gamma Gamma1,
  add_copies N (forall A) Gamma1 Gamma2.
amplify_forall N (A::Gamma) (A::Gamma1) :-
  amplify_forall N Gamma Gamma1.

amplify_exists N nil nil.
amplify_exists N ((exists A)::Delta) Delta2 :-
  amplify_exists N Delta Delta1,
  add_copies N (exists A) Delta1 Delta2.
amplify_exists N (A::Delta) (A::Delta1) :-
  amplify_exists N Delta Delta1.

amplify 1 Seq Seq :- !.
amplify N (Gamma1 --> Delta1) (Gamma2 --> Delta2) :-
  amplify_forall N Gamma1 Gamma2,
  amplify_exists N Delta1 Delta2.

nprove N Q Seq1 :-
  nl, writesans "Attempting to prove the following sequent at amplification ",
  write N, nl, write Seq1, nl,
  amplify N Seq1 Seq2,
  Q >-2 Seq2,
  writesans "successful".
nprove N Q Seq :-
  M is (N + 1), nprove M Q Seq.
