module ninormal.

% This module contains a specification of a theorem prover for the NJ
% natural deduction proof system for first-order logic as in Prawitz 1965.
% Only normal proofs are constructed.

import  nprf.

type    #       nprf -> form -> o.
type    #e      nprf -> form -> o.

infix   #       4.
infix   #e      4.

P # A :- P #e A.

(and_i P1 P2) # (A and B) :- P1 # A, P2 # B.
(or_i1 P) # (A or B) :- P # A.
(or_i2 P) # (A or B) :- P # B.
(imp_i P) # (A imp B) :- pi PA\ ((PA #e A) => ((P PA) # B)).
(neg_i P) # (neg A) :- pi PA\ ((PA #e A) => ((P PA) # false)).
(forall_i P) # (forall A) :- pi Y\ ((P Y) # (A Y)).
(exists_i T P) # (exists A) :- P # (A T).
(false_i P) # A :- P # false.

PC # C :- P #e (A and B),
          (((and_e1 B P) #e A) => (((and_e2 A P) #e B) => (PC # C))).

PC # C :- P2 #e (A imp B), P1 # A,
          (((imp_e A P1 P2) #e B) => (PC # C)).

PC # C :- P #e (forall A),
          (((forall_e T A P) #e (A T)) => (PC # C)).

(neg_e A P1 P2) # false :- P1 # A, P2 #e (neg A).

(or_e A B P P1 P2) # C :- P #e (A or B),
                          pi PA\ ((PA #e A) => ((P1 PA) # C)),
                          pi PB\ ((PB #e B) => ((P2 PB) # C)).

(exists_e A P1 P2) # B :- P1 #e (exists A),
                          pi Y\ (pi P\ ((P #e (A Y)) => ((P2 Y P) # B))).
